/* eslint-disable @typescript-eslint/ban-ts-comment */
/* eslint-disable @typescript-eslint/no-explicit-any */
import fs from "fs";
import { Prisma, SubCategory } from "@prisma/client";
import { paginationHelpers } from "../../../helpers/paginationHelper";
import { IGenericResponse } from "../../../interfaces/common";
import { IPaginationOptions } from "../../../interfaces/pagination";
import prisma from "../../../shared/prisma";
import httpStatus from "http-status";
import ApiError from "../../../errors/ApiError";
import { ISubCategoryFilterRequest, ISubCategoryRequest, ISubCategoryUpdateRequest } from "./subCategory.interface";
import { IUploadFile } from "../../../interfaces/file";
import {
  SubCategoryRelationalFields,
  SubCategoryRelationalFieldsMapper,
  SubCategorySearchableFields,
} from "./subCategory.constants";
import { Request } from "express";
import { errorLogger } from "../../../shared/logger";
import { subCategoryUtils } from "./subCategory.utils";

// modules

// !----------------------------------Create New Category--------------------------------------->>>
const addSubCategory = async (req: Request): Promise<SubCategory> => {
  //@ts-ignore
  const file = req.file as IUploadFile;

  const filePath = file?.path?.substring(13);

  //@ts-ignore
  const data = req.body as ISubCategoryRequest;

  if (!data.subCategoryName) {
    throw new ApiError(httpStatus.BAD_REQUEST, "Sub Category Name is required");
  }

  if (!filePath) {
    throw new ApiError(httpStatus.BAD_REQUEST, "Sub Category Image is required");
  }

  if (!data.categoryId) {
    throw new ApiError(httpStatus.BAD_REQUEST, "Category Id is required");
  }

  const isExistCategory = await prisma.category.findUnique({
    where: {
      categoryId: data.categoryId,
    },
  });

  if (!isExistCategory) throw new ApiError(httpStatus.BAD_REQUEST, "Category Not Found");

  const result = await prisma.$transaction(async (transactionClient) => {
    const newSubCategory = {
      subCategoryName: data.subCategoryName,
      subCategoryHref: data?.subCategoryName.replace(/[,\s&]+/g, "-").toLowerCase(),
      subCategoryImg: filePath,
      categoryId: isExistCategory.categoryId,
    };

    // Replace spaces with dashes in categoryName

    const isExistSubCategoryName = await transactionClient.subCategory.findFirst({
      where: {
        OR: [
          {
            subCategoryName: data?.subCategoryName,
          },
          {
            subCategoryHref: newSubCategory?.subCategoryHref,
          },
        ],
      },
    });

    if (isExistSubCategoryName)
      throw new ApiError(httpStatus.BAD_REQUEST, "Sub Category Name Already Exists, Try a new one!");

    const createdSubCategory = await transactionClient.subCategory.create({
      data: newSubCategory,
    });

    return createdSubCategory;
  });
  if (!result) {
    throw new ApiError(httpStatus.BAD_REQUEST, "Sub Category creation failed");
  }
  return result;
};

// !----------------------------------get all Category---------------------------------------->>>
const getSubCategory = async (
  filters: ISubCategoryFilterRequest,
  options: IPaginationOptions,
): Promise<IGenericResponse<SubCategory[]>> => {
  // Calculate pagination options
  const { limit, page, skip } = paginationHelpers.calculatePagination(options);

  // Destructure filter properties
  const { searchTerm, ...filterData } = filters;

  // Define an array to hold filter conditions
  const andConditions: Prisma.SubCategoryWhereInput[] = [];

  // Add search term condition if provided

  if (searchTerm) {
    andConditions.push({
      OR: SubCategorySearchableFields.map((field: any) => ({
        [field]: {
          contains: searchTerm,
          mode: "insensitive",
        },
      })),
    });
  }

  // Add filterData conditions if filterData is provided
  if (Object.keys(filterData).length > 0) {
    andConditions.push({
      AND: Object.keys(filterData).map((key) => {
        if (SubCategoryRelationalFields.includes(key)) {
          return {
            [SubCategoryRelationalFieldsMapper[key]]: {
              subCategoryName: (filterData as any)[key],
            },
          };
        } else {
          return {
            [key]: {
              equals: (filterData as any)[key],
            },
          };
        }
      }),
    });
  }

  // Create a whereConditions object with AND conditions
  const whereConditions: Prisma.SubCategoryWhereInput = andConditions.length > 0 ? { AND: andConditions } : {};

  // Retrieve Courier with filtering and pagination
  const result = await prisma.subCategory.findMany({
    include: {
      _count: true,
      product: true,
      category: true,
    },
    where: whereConditions,
    skip,
    take: limit,
    orderBy: options.sortBy && options.sortOrder ? { [options.sortBy]: options.sortOrder } : { createdAt: "desc" },
  });

  // Count total matching orders for pagination
  const total = await prisma.subCategory.count({
    where: whereConditions,
  });

  // Calculate total pages
  const totalPage = Math.ceil(total / limit);

  return {
    meta: {
      page,
      limit,
      total,
      totalPage,
    },
    data: result,
  };
};

// !----------------------------------get Single Courier---------------------------------------->>>
const getSingleSubCategory = async (subCategoryHref: string): Promise<SubCategory | null> => {
  if (!subCategoryHref) {
    throw new ApiError(httpStatus.BAD_REQUEST, "Sub Category Id is required");
  }

  const result = await prisma.subCategory.findUnique({
    where: {
      subCategoryHref,
    },
    include: {
      product: true,
      _count: {
        select: {
          product: true,
        },
      },
    },
  });

  if (!result) {
    throw new ApiError(httpStatus.NOT_FOUND, "Sub Category Not Found!!");
  }

  return result;
};

// !----------------------------------get Single Courier---------------------------------------->>>
const getSingleSubCategoryByName = async (subCategoryHref: string): Promise<any | null> => {
  if (!subCategoryHref) {
    throw new ApiError(httpStatus.BAD_REQUEST, "Sub Category Name is required");
  }

  const result = await prisma.subCategory.findUnique({
    where: {
      subCategoryHref,
    },
    select: {
      product: true,
    },
  });

  if (!result) {
    throw new ApiError(httpStatus.NOT_FOUND, "Sub Category Not Found!!");
  }

  return result;
};

// !----------------------------------Update Courier---------------------------------------->>>
const updateSubCategory = async (subCategoryId: string, req: Request): Promise<SubCategory> => {
  const file = req.file as IUploadFile;
  const filePath = file?.path?.substring(13);

  const { subCategoryName, categoryId, oldFilePath } = req.body as ISubCategoryUpdateRequest;

  // deleting old style Image
  const oldFilePaths = "uploads/" + oldFilePath;
  if (oldFilePath !== undefined && filePath !== undefined) {
    // @ts-ignore
    fs.unlink(oldFilePaths, (err) => {
      if (err) {
        errorLogger.error("Error deleting old file");
      }
    });
  }

  const updatedDetails: Partial<ISubCategoryUpdateRequest> = {
    subCategoryName,
    categoryId,
    subCategoryImg: filePath,
  };

  if (subCategoryName) updatedDetails["subCategoryHref"] = subCategoryName.replace(/[,\s&]+/g, "-").toLowerCase();

  const newSubCategoryData: Partial<ISubCategoryUpdateRequest> = subCategoryUtils.updateSubCategoryData(updatedDetails);

  const result = await prisma.$transaction(async (transactionClient) => {
    const existingSubCategory = await transactionClient.subCategory.findUnique({
      where: {
        subCategoryId,
      },
    });

    if (!existingSubCategory) {
      throw new ApiError(httpStatus.NOT_FOUND, "Sub Category Not Found!!");
    }

    if (categoryId) {
      const isCategoryExist = await transactionClient.category.findUnique({
        where: {
          categoryId,
        },
      });

      if (!isCategoryExist) {
        throw new ApiError(httpStatus.NOT_FOUND, "Category Not Found!!");
      }
    }

    // const updatedDetails: any = {
    //   subCategoryName: subCategoryName,
    //   subCategoryImg: filePath,
    // };
    // if (categoryId) updatedDetails['categoryId'] = categoryId;

    const updatedSubCategory = await transactionClient.subCategory.update({
      where: {
        subCategoryId,
      },
      data: newSubCategoryData,
    });

    return updatedSubCategory;
  });

  if (!result) {
    throw new ApiError(httpStatus.BAD_REQUEST, "Failed to update Sub Category Information");
  }

  return result;
};

const deleteSubCategory = async (subCategoryId: string): Promise<SubCategory> => {
  if (!subCategoryId) {
    throw new ApiError(httpStatus.BAD_REQUEST, "Sub Category Id is required");
  }

  const result = await prisma.subCategory.delete({
    where: {
      subCategoryId,
    },
  });

  return result;
};

export const SubCategoryService = {
  addSubCategory,
  getSubCategory,
  getSingleSubCategory,
  updateSubCategory,
  deleteSubCategory,
  getSingleSubCategoryByName,
};
